/*
 *      Copyright 2012  Samsung Electronics Co., Ltd
 *
 *      Licensed under the Flora License, Version 1.0 (the "License");
 *      you may not use this file except in compliance with the License.
 *      You may obtain a copy of the License at
 *
 *              http://www.tizenopensource.org/license
 *
 *      Unless required by applicable law or agreed to in writing, software
 *      distributed under the License is distributed on an "AS IS" BASIS,
 *      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *      See the License for the specific language governing permissions and
 *      limitations under the License.
 */

/**
 *	Module responsible for managing running apps
 */
var apps = (function() {
	/** @private */ var container = null;
	
	/** @private Makes list item
	 *  @param {String} name Name of the application
	 *  @param {String} icon path to the icon of application
	 *  @param {number} id ContextId of application
	 *  @returns {String} list item 
	 */
	var makeListItem = function(name, icon, id) {
		return "<li class=\"ui-li-has-multiline\" data-app-id=\"" + id + "\">" + "<img src=\"" + icon + "\"><h3>" + name + "</h3><p>touch to kill an app</p>";
	};

	return {
		/**
		 * Sets a list container for the list of applications 
		 * @param {String} containerId id of the list container element
		 */
		init : function(containerId) {
			container = $(containerId);
		},
		/**
		 * Gets list of running applications and put it to container
		 */
		getRunningAppsList : function() {
			var that = this;
			var hMakeListItem = makeListItem;
			var hContainer = container;
			tizen.application.getAppsContext(function(contexts) {
				hContainer.html('');
				var appInfo, item;
				for ( var i = 0; i < contexts.length; ++i) {
					appInfo = tizen.application.getAppInfo(contexts[i].appId);
					if (appInfo.show) {
						item = hMakeListItem(appInfo.name, appInfo.iconPath, contexts[i].id);
						hContainer.append(item);
					}
				}
				hContainer.listview('refresh');
				hContainer.find('li').unbind().bind({
					click : function(event) {
						var id = this.getAttribute('data-app-id');
						that.killApp(id);
					}
				});
			}, function() {
				console.log("getAppsContext() error");
			});
		},
		/**
		 * When user confirm kills the application with given contextId and if success lunch callback function
		 * @param {number} contextId ContextId of application to be killed
		 */
		killApp : function(contextId) {
			var that = this;
			if (confirm("Are you sure to exit this app?")) {
				tizen.application.kill(contextId, function() {	
					that.getRunningAppsList();
				}, function() {
					console.log("killApp() error");
				});
			}
		}
	};
})();
/**
 * Module responsible for showing performance 
 */
var performance = (function() {
	var processor = null;
	var storage = null;
	var power = null;

	return {
		/**
		 * Gets handlers for CPU, Storage and Power meters
		 */
		init : function() {
			processor = document.getElementById('processor');
			storage = document.getElementById('storage');
			power = document.getElementById('power');
		},
		/**
		 * Get CPU, Power and Storage properties and put them into corresponding meters
		 */
		getSystemInfoValues : function() {
			var hProcessor = processor;
			var hStorage = storage;
			var hPower = power;
			var hSetCpuMeter = this.setCpuMeter;
			var hSetStorageMeter = this.setStorageMeter;
			var hSetPowerMeter = this.setPowerMeter;
			if (tizen.systeminfo.isSupported('Cpu')) {
				tizen.systeminfo.getPropertyValue('Cpu', function(property) {
					hSetCpuMeter(property, hProcessor);
				});
				tizen.systeminfo.addPropertyValueChangeListener('Cpu', function(property) {
					hSetCpuMeter(property, hProcessor);
				}, function() {
					console.log('addPropertyValueChangeListener() error');
				});
			}
			if (tizen.systeminfo.isSupported('Storage')) {
				tizen.systeminfo.getPropertyValue('Storage', function(property) {
					hSetStorageMeter(property, hStorage);
				});
				tizen.systeminfo.addPropertyValueChangeListener('Storage', function(property) {
					hSetStorageMeter(property, hStorage);
				}, function() {
					console.log('addPropertyValueChangeListener() error');
				});
			}
			if (tizen.systeminfo.isSupported('Power')) {
				tizen.systeminfo.getPropertyValue('Power', function(property) {
					hSetPowerMeter(property, hPower);
				});
				tizen.systeminfo.addPropertyValueChangeListener('Power', function(property) {
					hSetPowerMeter(property, hPower);
				}, function() {
					console.log('addPropertyValueChangeListener() error');
				});
			}
		},
		/**
		 * Sets the CPU meter to the given value
		 * @param {Object} cpuInfo SystemInfoCpu object
		 * @param {DOM meter} cpuMeter handler to the CPU meter 
		 */
		setCpuMeter : function(cpuInfo, cpuMeter) {
			tizen.logger.info(cpuInfo.load);
			cpuMeter.value = cpuInfo.load;
		},
		/**
		 * Sets the Storage meter to the value from the first storage device
		 * @param {Objec} storageInfo SystemInfoStorage object
		 * @param {DOM meter} storageMeter handler to the storage meter 
		 * @returns
		 */
		setStorageMeter : function(storageInfo, storageMeter) {
			if (storageInfo.length > 0) {
				var main = storageInfo[0];
				storageMeter.max = main.capacity;
				storageMeter.value = main.capacity - main.availableCapacity;
			}
		},
		/**
		 * Sets the Power meter to the given value
		 * @param {Objec} powerInfo SystemInfoPower object
		 * @param {DOM meter} powerMeter handler to the power meter
		 * @returns
		 */
		setPowerMeter : function(powerInfo, powerMeter) {
			powerMeter.value = powerInfo.level;
		}
	};
})();

// Initialize function
var init = function() {
	tizen.logger = tizen.logger({
		logLevel : 3
	});
	apps.init("#apps_list");
	apps.getRunningAppsList();
	performance.init();
	performance.getSystemInfoValues();

	/**
	 * When application goes from background to foreground refresh list of running applications and gets the system values  
	 */
	tizen.application.visibility.onApplicationVisible(function() {
		apps.getRunningAppsList();
		performance.getSystemInfoValues();
	});
};
$(document).ready(init);
